### -*-python-*-
###
### Cryptographic primitives
###
### (c) 2013 Mark Wooding
###

###----- Licensing notice ---------------------------------------------------
###
### This file is part of Chopwood: a password-changing service.
###
### Chopwood is free software; you can redistribute it and/or modify
### it under the terms of the GNU Affero General Public License as
### published by the Free Software Foundation; either version 3 of the
### License, or (at your option) any later version.
###
### Chopwood is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU Affero General Public License for more details.
###
### You should have received a copy of the GNU Affero General Public
### License along with Chopwood; if not, see
### <http://www.gnu.org/licenses/>.

from struct import pack, unpack

###--------------------------------------------------------------------------
### The core of MD5.

def U32(x): return x&0xffffffff
def FF(x, y, z): return (x&y) | (~x&z)
def GG(x, y, z): return (z&x) | (~z&y)
def HH(x, y, z): return x^y^z
def II(x, y, z): return y^(x|~z)
def rot(x, n): return U32((x << n) | (x >> 32 - n))
MD5_INIT = '0123456789abcdeffedcba9876543210'.decode('hex')
def compress_md5(buf, st = MD5_INIT):
  """
  The MD5 compression function, in pure Python.

  This is about as small as I could make it.  Apply the MD5 compression
  function to BUF, using the initial state ST (defaults to the standard
  initialization vector); return the new state as the function value.
  """
  a, b, c, d = unpack('<4L', st)
  aa, bb, cc, dd = a, b, c, d
  x = xx = unpack('<16L', buf)
  for f, i, r, k in [(FF,  0,  7, 0xd76aa478), (FF,  1, 12, 0xe8c7b756),
                     (FF,  2, 17, 0x242070db), (FF,  3, 22, 0xc1bdceee),
                     (FF,  4,  7, 0xf57c0faf), (FF,  5, 12, 0x4787c62a),
                     (FF,  6, 17, 0xa8304613), (FF,  7, 22, 0xfd469501),
                     (FF,  8,  7, 0x698098d8), (FF,  9, 12, 0x8b44f7af),
                     (FF, 10, 17, 0xffff5bb1), (FF, 11, 22, 0x895cd7be),
                     (FF, 12,  7, 0x6b901122), (FF, 13, 12, 0xfd987193),
                     (FF, 14, 17, 0xa679438e), (FF, 15, 22, 0x49b40821),
                     (GG,  1,  5, 0xf61e2562), (GG,  6,  9, 0xc040b340),
                     (GG, 11, 14, 0x265e5a51), (GG,  0, 20, 0xe9b6c7aa),
                     (GG,  5,  5, 0xd62f105d), (GG, 10,  9, 0x02441453),
                     (GG, 15, 14, 0xd8a1e681), (GG,  4, 20, 0xe7d3fbc8),
                     (GG,  9,  5, 0x21e1cde6), (GG, 14,  9, 0xc33707d6),
                     (GG,  3, 14, 0xf4d50d87), (GG,  8, 20, 0x455a14ed),
                     (GG, 13,  5, 0xa9e3e905), (GG,  2,  9, 0xfcefa3f8),
                     (GG,  7, 14, 0x676f02d9), (GG, 12, 20, 0x8d2a4c8a),
                     (HH,  5,  4, 0xfffa3942), (HH,  8, 11, 0x8771f681),
                     (HH, 11, 16, 0x6d9d6122), (HH, 14, 23, 0xfde5380c),
                     (HH,  1,  4, 0xa4beea44), (HH,  4, 11, 0x4bdecfa9),
                     (HH,  7, 16, 0xf6bb4b60), (HH, 10, 23, 0xbebfbc70),
                     (HH, 13,  4, 0x289b7ec6), (HH,  0, 11, 0xeaa127fa),
                     (HH,  3, 16, 0xd4ef3085), (HH,  6, 23, 0x04881d05),
                     (HH,  9,  4, 0xd9d4d039), (HH, 12, 11, 0xe6db99e5),
                     (HH, 15, 16, 0x1fa27cf8), (HH,  2, 23, 0xc4ac5665),
                     (II,  0,  6, 0xf4292244), (II,  7, 10, 0x432aff97),
                     (II, 14, 15, 0xab9423a7), (II,  5, 21, 0xfc93a039),
                     (II, 12,  6, 0x655b59c3), (II,  3, 10, 0x8f0ccc92),
                     (II, 10, 15, 0xffeff47d), (II,  1, 21, 0x85845dd1),
                     (II,  8,  6, 0x6fa87e4f), (II, 15, 10, 0xfe2ce6e0),
                     (II,  6, 15, 0xa3014314), (II, 13, 21, 0x4e0811a1),
                     (II,  4,  6, 0xf7537e82), (II, 11, 10, 0xbd3af235),
                     (II,  2, 15, 0x2ad7d2bb), (II,  9, 21, 0xeb86d391)]:
    b, c, d, a = U32(rot(U32(a + f(b, c, d) + x[i] + k), r) + b), b, c, d
  return pack('<4L', U32(a + aa), U32(b + bb), U32(c + cc), U32(d + dd))

###----- That's all, folks --------------------------------------------------
