### -*-python-*-
###
### Database maintenance
###
### (c) 2013 Mark Wooding
###

###----- Licensing notice ---------------------------------------------------
###
### This file is part of Chopwood: a password-changing service.
###
### Chopwood is free software; you can redistribute it and/or modify
### it under the terms of the GNU Affero General Public License as
### published by the Free Software Foundation; either version 3 of the
### License, or (at your option) any later version.
###
### Chopwood is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU Affero General Public License for more details.
###
### You should have received a copy of the GNU Affero General Public
### License along with Chopwood; if not, see
### <http://www.gnu.org/licenses/>.

from __future__ import with_statement

import config as CONF; CFG = CONF.CFG
import subcommand as SC
import util as U

###--------------------------------------------------------------------------
### Opening the database.

## Current database schema version.
DB_VERSION = 0

def opendb():
  """Open connections to the master database, updating it if necessary."""

  global DB

  ## Open the database.
  dbmod, dbargs = CFG.DB
  DB = U.SimpleDBConnection(dbmod, dbargs)

  ## Find out the current version.
  try:
    DB.execute("SELECT version FROM meta")
  except DB.Error:
    ver = 0
  else:
    ver, = DB.fetchone()
    if ver > DB_VERSION:
      raise ExpectedError, (500, "Database schema version not understood.")

  ## In future, there will be an attempt to upgrade databases with old
  ## schemata to the current version.  But not yet.
  pass

###--------------------------------------------------------------------------
### Database setup.

@SC.subcommand('setup', ['admin'], 'Initialize the master database.')
def cmd_setup():
  script = """
        CREATE TABLE users (
                user VARCHAR(32) PRIMARY KEY NOT NULL,
                passwd TEXT NOT NULL DEFAULT('*'),
                email TEXT);
        CREATE TABLE services (
                user VARCHAR(32) NOT NULL,
                service VARCHAR(32) NOT NULL,
                alias VARCHAR(32),
                PRIMARY KEY (user, service),
                FOREIGN KEY (user) REFERENCES users(user)
                        ON UPDATE CASCADE
                        ON DELETE CASCADE);
        CREATE TABLE meta (
                version INTEGER NOT NULL);
        CREATE TABLE secrets (
                stamp INTEGER PRIMARY KEY NOT NULL,
                secret TEXT NOT NULL);
  """
  with DB:
    for cmd in script.split(';'):
      if not cmd.isspace():
        DB.execute(cmd)
    DB.execute("INSERT INTO meta(version) VALUES ($version)",
               version = DB_VERSION)

###----- That's all, folks --------------------------------------------------
